#include <errno.h>
#include <fcntl.h>
#include <getopt.h>
#include <linux/rtc.h>
#include <poll.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <termios.h>
#include <time.h>
#include <unistd.h>

#define RTC_ALARM_TIME_CONF			"/etc/rtc_alarm.conf"
#define SYS_POWER_STATE_PATH		"/sys/class/rtc/rtc0/wakealarm"

#define msleep(x) usleep((x)*1000)

struct up_file_t
{
    char sym[64];
    char dst[128];
};

char *trim(char *line, int str_len)
{
    int pos = 0;
    char *orig = line;
    char tmp[2048];

    if(NULL == line)
    {
        return NULL;
    }
    line[str_len - 1] = 0;
    memset(tmp, 0x00, sizeof(tmp));

    while(*line != '\0')
    {
		//去除空格、回车、换行、制表符
        if((*line == ' ') || (*line == '\t') || (*line == '\r') || (*line == '\n')) 
        {

        }
        else
        {
            tmp[pos++] = *line;
            if(sizeof(tmp) <= pos)
            {
                tmp[sizeof(tmp) - 1] = 0;
                break;
            }
        }
        line++;
    }

    int size = strlen(orig);
    if(str_len < size)
    {
        size = str_len;
    }
    memset(orig, 0x00, size);
    memcpy(orig, tmp, pos);
    return orig;
}

int parse_line(char *line, struct up_file_t *uf)
{
    char tline[1024] = { 0 };

    strncpy(tline, line, sizeof(tline));

    char *sym = tline;
    char *end = strstr(tline, "=");
    if(end == NULL)
    {
        printf("config file, ignore line: %s\n", line);
        return -1;
    }

    char *dst = end;
    dst++;
    *end = 0;

    if(dst[0] == 0)
    {
        printf("config file, ignore line: %s\n", line);
        return -1;
    }

    if(NULL != uf)//
    {
        strncpy(uf->sym, sym, sizeof(uf->sym));
        strncpy(uf->dst, dst, sizeof(uf->dst));
    }

    return 0;
}

int parse_conf(char *file, struct up_file_t *ufa, int max_cnt)
{
    int fcnt = 0;
    char buf[1024] = { 0 };
    FILE *fp = NULL;

    fp = fopen(file, "r"); //打开文件
    if(fp == NULL)
    {
        printf("open file %s: %s\n", file, strerror(errno));
        return -1;
    }

    while(fgets(buf, 1024, fp)) //读取每一行
    {
        trim(buf, sizeof(buf));

		struct up_file_t uf;

		if(parse_line(buf, &uf) == 0) //解析行数据
		{
			ufa[fcnt++] = uf;
			if(fcnt >= max_cnt)
			{
				break;
			}
		}
    }
    fclose(fp);
    return fcnt;
}


void alarm_feed(int timeout)
{
	int fd = -1;
	char buf[256] = { 0 };
	ssize_t ss;

	fd = open(SYS_POWER_STATE_PATH, O_WRONLY);
	if (fd < 0)
		goto nothing;
	
	strcpy(buf,"0");
	ss = write(fd, buf,strlen(buf));
	if (ss <= 0)
		goto nothing;
	
	sprintf(buf, "+%d",timeout);
	ss = write(fd, buf,strlen(buf));
	if (ss <= 0)
		goto nothing;

nothing:
	if (fd >= 0)
		close(fd);
}


int main(int argc, char **argv)
{
	int num = 0;
	int timeout = 60;
	time_t sec_timestamp;
	struct up_file_t uf[4];
	
	if(access(RTC_ALARM_TIME_CONF, F_OK) == 0)
	{
		num = parse_conf(RTC_ALARM_TIME_CONF, uf, 64);
	
		for(int index = 0; index < num; index++)
		{
			if(strcmp(uf[index].sym, "alarm_timeout") == 0) 
			{
				timeout = atoi(uf[index].dst) * 60;
				if(timeout < 60)
				{
					timeout = 60;
				}
				break;
			}
		}
	}

    time(&sec_timestamp);

	if(sec_timestamp%60 < 30)
	{
		alarm_feed(timeout);
	}
	else
	{
		alarm_feed(timeout+30);		
	}

	while(1)
	{
		msleep(500);
		
		time(&sec_timestamp);
		
		if(sec_timestamp%60 > 55)
		{
			alarm_feed(timeout+30);
		}
	}
	
	return 0;
}